__author__ = "Lukas Mahler"
__version__ = "0.0.0"
__date__ = "06.05.2022"
__email__ = "m@hler.eu"
__status__ = "Development"


# Default
import json

# Custom
import requests
import socketio


class Swapgg:

    def __init__(self, log, config):
        self.apikey = config['Swap']['apikey']
        self.headers = {'Content-type': 'application/json', 'Authorization': f'{self.apikey}'}
        self.sio = socketio.Client()
        self.log = log

        self.baseurl = "https://market-api.swap.gg/v1/"
        self.socketToken = None
        self.imglnk = None
        self.requested = []

        self._getSocketToken()

    def _getSocketToken(self):
        endpoint = "user/websocket"
        r = requests.get(self.baseurl + endpoint, headers=self.headers)
        if r.status_code == 200:
            response = r.json()
            if response['status'] == "OK":
                self.socketToken = response['result']['token']
                self._initSocket()
            else:
                self.log.pipeOut({response['status']}, lvl='ERROR')
        else:
            self.log.pipeOut({r.status_code}, lvl='ERROR')

    def _initSocket(self):
        url = "https://market-ws.swap.gg/"
        self.sio.connect(url)
        self.sio.emit('auth', self.socketToken)
        self.sio.on("screenshot:ready", self.passScreenshot)

    def _post(self, endpoint: str, payload: dict) -> requests.Response:
        url = self.baseurl + endpoint
        self.log.pipeOut(f"Posting to [{url}] / Payload [{payload}]", lvl='DEBUG')
        req = requests.post(url, headers=self.headers, data=json.dumps(payload))
        self.log.pipeOut(f"Returned [{req.status_code}] - {req.text}", lvl='DEBUG')

        return req

    def getScreenshot(self, inspect: str):
        self.imglnk = ""
        endpoint = "screenshot"

        # Rebuild inspect link
        inspectlnk = inspect.replace("%20", " ")

        payload = {"inspectLink": inspectlnk}
        req = self._post(endpoint, payload)
        response = req.json()
        self.log.pipeOut(response)

        if response['status'] == "OK":
            rslt = response['result']

            # Looks like swap provides no more requestId. sadge
            # rid = rslt['requestId']
            rid = inspectlnk

            if rslt['state'] == "COMPLETED":
                self.imglnk = response['result']['imageLink']
                self.log.pipeOut(self.imglnk)
            elif rslt['state'] == "IN_QUEUE":
                self.requested.append(rid)
                pass
        else:
            self.log.pipeOut({response['status']}, lvl='ERROR')

    def passScreenshot(self, data: dict):
        # print(data)
        if 'inspectLink' in data:
            if data['inspectLink'] in self.requested:
                self.imglnk = data['imageLink']
                self.requested.remove(data['inspectLink'])
        else:
            self.log.pipeOut(data, lvl='WARNING')


if __name__ == '__main__':
    exit()
